using System;
using System.Diagnostics;
using System.Reflection;
using System.Security.Permissions;
using System.Collections.Generic;
using System.Text;

namespace Driver
{
    class CasGen
    {
        internal static void Run()
        { 
            Type[] ts = new Type[] {
                typeof(System.Configuration.ConfigurationPermissionAttribute),
                typeof(System.Net.DnsPermissionAttribute),
                typeof(System.Security.Permissions.GacIdentityPermissionAttribute),
                typeof(System.Transactions.DistributedTransactionPermissionAttribute),
                typeof(System.Data.Common.DBDataPermissionAttribute),
                typeof(System.Diagnostics.EventLogPermissionAttribute),
                typeof(System.Diagnostics.PerformanceCounterPermissionAttribute),
                typeof(System.DirectoryServices.DirectoryServicesPermissionAttribute),
                typeof(System.Drawing.Printing.PrintingPermissionAttribute),
                typeof(System.Messaging.MessageQueuePermissionAttribute),
                typeof(System.Net.Mail.SmtpPermissionAttribute),
                typeof(System.Net.NetworkInformation.NetworkInformationPermissionAttribute),
                typeof(System.Net.SocketPermissionAttribute),
                typeof(System.Net.WebPermissionAttribute),
                typeof(System.Security.Permissions.DataProtectionPermissionAttribute),
                typeof(System.Security.Permissions.EnvironmentPermissionAttribute),
                typeof(System.Security.Permissions.FileDialogPermissionAttribute),
                typeof(System.Security.Permissions.FileIOPermissionAttribute),
                typeof(System.Security.Permissions.HostProtectionAttribute),
                typeof(System.Security.Permissions.IsolatedStoragePermissionAttribute),
                typeof(System.Security.Permissions.KeyContainerPermissionAttribute),
                typeof(System.Security.Permissions.PermissionSetAttribute),
                typeof(System.Security.Permissions.PrincipalPermissionAttribute),
                typeof(System.Security.Permissions.PublisherIdentityPermissionAttribute),
                typeof(System.Security.Permissions.ReflectionPermissionAttribute),
                typeof(System.Security.Permissions.RegistryPermissionAttribute),
                typeof(System.Security.Permissions.SecurityPermissionAttribute),
                typeof(System.Security.Permissions.SiteIdentityPermissionAttribute),
                typeof(System.Security.Permissions.StorePermissionAttribute),
                typeof(System.Security.Permissions.StrongNameIdentityPermissionAttribute),
                typeof(System.Security.Permissions.UIPermissionAttribute),
                typeof(System.Security.Permissions.UrlIdentityPermissionAttribute),
                typeof(System.Security.Permissions.ZoneIdentityPermissionAttribute),
                typeof(System.ServiceProcess.ServiceControllerPermissionAttribute),
                typeof(System.Web.AspNetHostingPermissionAttribute)
            };

            foreach (Type t in ts)
            {
                string name = t.Name.Substring(0, (t.Name.IndexOf("PermissionAttribute") != -1 ? t.Name.IndexOf("PermissionAttribute") : t.Name.IndexOf("Attribute")));

                Debug.WriteLine("/// <summary>");
                Debug.WriteLine(string.Format("/// Builds a <see cref=\"{0}\"/> attribute.", t.FullName));
                Debug.WriteLine("/// </summary>");
                Debug.WriteLine("[Serializable, CLSCompliant(true)]");
                Debug.WriteLine(string.Format("public sealed class {0} : Permission", name));
                Debug.WriteLine("{");
                Debug.WriteLine("    /// <summary>");
                Debug.WriteLine(string.Format("    /// An instance of <see cref=\"{0}\"/> representing the default values.", name));
                Debug.WriteLine("    /// See <see cref=\"Permission\"/> for an explanation about the defaults.");
                Debug.WriteLine("    /// </summary>");
                Debug.WriteLine(string.Format("    public static readonly {0} Empty = new {0}();", name));
                Debug.WriteLine(string.Empty);
                Debug.WriteLine("    /// <summary>");
                Debug.WriteLine("    /// Returns the attribute type associated with this permission.");
                Debug.WriteLine("    /// </summary>");
                Debug.WriteLine("    protected internal override Type Type");
                Debug.WriteLine("    {");
                Debug.WriteLine("        get");
                Debug.WriteLine("        {");
                Debug.WriteLine(string.Format("            return typeof({0});", t.FullName));
                Debug.WriteLine("        }");
                Debug.WriteLine("    }");

                foreach (PropertyInfo pi in t.GetProperties(BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly))
                {
                    if (pi.Name != "Action" && pi.Name != "Unrestricted")
                    {
                        Debug.WriteLine(string.Empty);
                        Debug.WriteLine(string.Format("    private static readonly string {0}ArgumentName = typeof({1}).GetProperty(\"{0}\").Name;", pi.Name, t.FullName));
                        Debug.WriteLine(string.Empty);

                        Debug.WriteLine("    /// <summary>");
                        Debug.WriteLine(string.Format("    /// Sets the value to be placed in <see cref=\"{0}.{1}\" />.", t.FullName, pi.Name));
                        Debug.WriteLine("    /// </summary>");
                        Debug.WriteLine(string.Format("    public {1} {0}", pi.Name, pi.PropertyType.FullName));
                        Debug.WriteLine("    {");
                        Debug.WriteLine("        set");
                        Debug.WriteLine("        {");
                        Debug.WriteLine(string.Format("            base.SetArgument({0}ArgumentName, value);", pi.Name));
                        Debug.WriteLine("        }");
                        Debug.WriteLine("    }");
                    }
                }

                Debug.WriteLine("}");

                Debug.WriteLine(string.Empty);
            }
        }
    }
}
